@@ -468,7 +468,7 @@ fitact <- function(dat, wt=NULL, reps=999, bw=NULL, adj=1, sample=c("none","data
 #' compareCkern(fPaca,fRat,reps=10)
 #' @export
 compareCkern <- function(fit1, fit2, reps=999){
-  if(class(fit1)!="actmod" | class(fit1)!="actmod")
+  if(!inherits(fit1, "actmod") | !inherits(fit2, "actmod"))
     stop("Input must be fitted activity models (class actmod)")
   bnd <- range(fit1@pdf[,1])
   if(!all(bnd==range(fit2@pdf[,1])))
@@ -519,7 +519,7 @@ compareCkern <- function(fit1, fit2, reps=999){
 #' compareAct(list(fPaca,fRat))
 #' @export
 compareAct <- function(fits)
-{ if(class(fits)!="list" | !all(unlist(lapply(fits,class))=="actmod"))
+{ if(!inherits(fits, "list") | !all(unlist(lapply(fits,inherits,"actmod"))))
     stop("fits must be a list of actmod objects")
   if(min(unlist(lapply(fits, function(x) length(x@act))))==1)
     stop("all input model fits must be boostrapped")
@@ -555,7 +555,7 @@ compareAct <- function(fits)
 #' compareTimes(fPaca, c(5.5,6,0.5,1))
 #' @export
 compareTimes <- function(fit, times)
-{ if(class(fit)!="actmod") stop("fit input must be an actmod object")
+{ if(!inherits(fit, "actmod")) stop("fit input must be an actmod object")
   if(!all(times>=0 & times<=2*pi)) stop("some times are <0 or >2*pi, expecting radian data")
 
   len <- length(times)
@@ -690,6 +690,7 @@ fitlincirc <- function(circdat, lindat, pCI=0.95, reps=10, res=512)
 #' @param add Logical defining whether to create a new plot (default) or add to an existing plot.
 #' @param xaxis List of plotting parameters to pass to axis command for x-axis plot (see axis for arguments).
 #' @param ... Additional arguments passed to internal plot call affecting only the plot frame and y axis. Modify x axis through xaxis.
+#' @return No return value, called to create a plot visualising an activity model.
 #' @examples
 #' data(BCItime)
 #' otm <- 2*pi*subset(BCItime, species=="ocelot")$time
@@ -845,6 +846,7 @@ plot.actmod <- function(x, xunit=c("clock","hours","radians"), yunit=c("frequenc
 #' @param tlim Numeric vector with two elements >=0 and <=1 defining the lower and upper limits at which to plot distributions; default plots the full range.
 #' @param fcol,flty,ncol,nlty Define line colour (\code{col}) and type (\code{lty}) for fitted (\code{f}) and null (\code{n}) distributions; input types as for \code{col} and \code{lty}, see \code{\link{par}}.
 #' @param ... Additional arguments passed to the inital plot construction, affecting axes and data plot symbols.
+#' @return No return value, called to create a plot visualising a linear-circular relationship.
 #' @export
 plot.lincircmod <- function(x, CircScale=2*pi, tlim=c(0,1), fcol="black", flty=1, ncol="red", nlty=2, ...){
 
@@ -881,8 +883,9 @@ plot.lincircmod <- function(x, CircScale=2*pi, tlim=c(0,1), fcol="black", flty=1
 #' Accepts data of class POSIXct, POSIXlt or character and returns the  time of day element as numeric (any date element is ignored).
 #'
 #' @param x A vector of POSIXct, POSIXlt or character format time data to convert.
-#' @param format A character string defining the time/date format if \code{x} is character format, see \code{strptime} for details. Ignored if \code{x} is not character.
 #' @param scale The scale on which to return times (see Value for options).
+#' @param ... arguments passed to as.POSIXlt
+#' @param tryFormats formats to try when converting date from character, passed to as.POSIXlt
 #' @return A vector of numeric times of day in units defined by the \code{scale} argument:
 #' radian, on the range [0, 2*pi];
 #' hours, on the range [0, 24];
@@ -890,25 +893,30 @@ plot.lincircmod <- function(x, CircScale=2*pi, tlim=c(0,1), fcol="black", flty=1
 #' @seealso \code{\link{strptime}}
 #' @examples
 #' data(BCItime)
-#' rtime <- gettime(BCItime$date, "%d/%m/%Y %H:%M")
-#' htime <- gettime(BCItime$date, "%d/%m/%Y %H:%M", "hour")
-#' ptime <- gettime(BCItime$date, "%d/%m/%Y %H:%M", "proportion")
+#' rtime <- gettime(BCItime$date)
+#' htime <- gettime(BCItime$date, "hour")
+#' ptime <- gettime(BCItime$date, "proportion")
 #' summary(rtime)
 #' summary(htime)
 #' summary(ptime)
 #' @export
-gettime <- function(x, format="%Y-%m-%d %H:%M:%S", scale=c("radian","hour","proportion")){
-  if(class(x)[1]=="character") x <- strptime(x, format, "UTC") else
-    if(class(x)[1]=="POSIXct") x <- as.POSIXlt(x) else
-      if(class(x)[1]!="POSIXlt") stop("x must be character or POSIXt class")
-
-    scale <- match.arg(scale)
-
-    res <- x$hour + x$min/60 + x$sec/3600
-    if(scale=="radian") res <- res*pi/12
-    if(scale=="proportion") res <- res/24
-    if(all(res==0, na.rm=T)) warning("All times are 0: may be just strptime default?")
-    res
+gettime <- function(x, scale=c("radian","hour","proportion"), ...,
+                    tryFormats=c("%Y-%m-%d %H:%M:%OS",
+                                 "%Y/%m/%d %H:%M:%OS",
+                                 "%Y:%m:%d %H:%M:%OS",
+                                 "%Y-%m-%d %H:%M",
+                                 "%Y/%m/%d %H:%M",
+                                 "%Y:%m:%d %H:%M",
+                                 "%Y-%m-%d",
+                                 "%Y/%m/%d",
+                                 "%Y:%m:%d")){
+  scale <- match.arg(scale)
+  x <- as.POSIXlt(x, tryFormats=tryFormats, ...)
+  res <- x$hour + x$min/60 + x$sec/3600
+  if(scale=="radian") res <- res*pi/12
+  if(scale=="proportion") res <- res/24
+  if(all(res==0, na.rm=T)) warning("All times are 0: may be just strptime default?")
+  res
 }
 
 
@@ -952,6 +960,72 @@ cmean <- function(x, ...){
   wrap(atan(Y/X) + ifelse(X<0,pi,0))
 }
 
+#' Calculates solar event times
+#'
+#' Calculates approximate times of sunrise and sunset and day lengths
+#' for given dates at given locations.
+#'
+#' @details Function adapted from https://www.r-bloggers.com/2014/09/seeing-the-daylight-with-r/
+#' @param date character, POSIX or Date format date/time value(s)
+#' @param lat,lon latitude and longitude in decimal degrees
+#' @param offset the time offset in hours relative to UTC (GMT) for results
+#' @param ... arguments passed to as.POSIXlt
+#' @param tryFormats formats to try when converting date from character, passed to as.POSIXlt
+#' @return A dataframe with columns sunrise and sunset (given in the timezone defined by offset) and daylength, all expressed in hours.
+#' @references Teets, D.A. 2003. Predicting sunrise and sunset times. The College Mathematics Journal 34(4):317-321.
+#' @examples
+#' data(BCItime)
+#' dat <- subset(BCItime, species=="ocelot")$date
+#' get_suntimes(dat, 9.156335, -79.847682, -5)
+#' @export
+get_suntimes <- function(date, lat, lon, offset, ...,
+                         tryFormats=c("%Y-%m-%d %H:%M:%OS",
+                                      "%Y/%m/%d %H:%M:%OS",
+                                      "%Y:%m:%d %H:%M:%OS",
+                                      "%Y-%m-%d %H:%M",
+                                      "%Y/%m/%d %H:%M",
+                                      "%Y:%m:%d %H:%M",
+                                      "%Y-%m-%d",
+                                      "%Y/%m/%d",
+                                      "%Y:%m:%d")){
+  nlat <- length(lat)
+  nlon <- length(lon)
+  ndat <- length(date)
+  if((nlat>1 & nlat!=ndat) | (nlon>1 & nlon!=ndat))
+    stop("lat and lon must have length 1 or the same length as date")
+  # Day of year
+  d <- as.POSIXlt(date, tryFormats=tryFormats, ...)$yday + 1
+  # Radius of the earth (km)
+  R <- 6378
+  # Radians between the xy-plane and the ecliptic plane
+  epsilon <- 23.45 * pi / 180
+  # Convert observer's latitude to radians
+  L <- lat * pi / 180
+  # Calculate offset of sunrise based on longitude (min)
+  # If lon is negative, then the mod represents degrees West of
+  # a standard time meridian, so timing of sunrise and sunset should
+  # be made later.
+  lon_h <- 24 * lon / 360
+  # The earth's mean distance from the sun (km)
+  r <- 149598000
+  theta <- 2 * pi / 365.25 * (d - 80)
+  zs <- r * sin(theta) * sin(epsilon)
+  rp <- sqrt(r^2 - zs^2)
+  acos_term <- (R-zs * sin(L)) / (rp * cos(L))
+  t0 <- suppressWarnings(1440 / (2*pi) * acos(acos_term))
+  # A kludge adjustment for the radius of the sun
+  that <- t0+5
+  # Adjust "noon" for the fact that the earth's orbit is not circular:
+  n <- 720 - 10 * sin(4 * pi * (d-80) / 365.25) + 8 * sin(2*pi*d / 365.25)
+  ## now sunrise and sunset are:
+  sunrise <- (n - that) / 60 - lon_h + offset
+  sunset <- (n + that) / 60 - lon_h + offset
+  daylength <- ifelse(acos_term < -1, 24,
+                      ifelse(acos_term > 1, 0,
+                             sunset-sunrise))
+  data.frame(sunrise=sunrise, sunset=sunset, daylength=daylength)
+}
+
 
 #' Transforms clock time to solar time anchored to sun rise and sunset times for a given location.
 #'
@@ -959,9 +1033,10 @@ cmean <- function(x, ...){
 #'
 #' @details Time zone \code{tz} should be expressed in numeric hours relative to UTC (GMT).
 #' @param dat A vector of character, POSIXct or POSIXlt date-time values.
-#' @param lat,long Single numeric values or numeric vectors the same length as \code{dat} giving site latitude and longitude in decimal format.
+#' @param lat,lon Single numeric values or numeric vectors the same length as \code{dat} giving site latitude and longitude in decimal format.
 #' @param tz A single numeric value or numeric vector same length as \code{dat} giving time zone (see Details).
-#' @param format A character string defining the time-date format if \code{dat} is character format, see \code{strptime} for details. Ignored if \code{x} is not character.
+#' @param ... arguments passed to as.POSIXlt
+#' @param tryFormats formats to try when converting date from character, passed to as.POSIXlt
 #' @return A list with elements:
 #' @return \code{input}: event input dates-times in POSIXlt format.
 #' @return \code{clock}: radian clock time data.
@@ -971,26 +1046,30 @@ cmean <- function(x, ...){
 #' @examples
 #' data(BCItime)
 #' subdat <- subset(BCItime, species=="ocelot")
-#' times <- solartime(subdat$date, 9.156335, -79.847682, -5, "%d/%m/%Y %H:%M")
+#' times <- solartime(subdat$date, 9.156335, -79.847682, -5)
 #' rawAct <- fitact(times$clock)
 #' avgAct <- fitact(times$solar)
 #' plot(rawAct)
 #' plot(avgAct, add=TRUE, data="n", tline=list(col="cyan"))
 #' @export
-solartime <- function(dat, lat, long, tz, format="%Y-%m-%d %H:%M:%S"){
-  if(class(dat)[1]=="character"){
-    dat <- strptime(dat, format, "UTC")
-    if(any(is.na(dat))) stop("Date conversion was at least partly unsuccessful - check formatting")
-  } else
-    if(!grepl("POSIX", class(dat)[1]))
-      stop("dat must be character or POSIXt class")
-  posdat <- list(lat,long,tz)
-  if(any(unlist(lapply(posdat, class)) != "numeric") |
+solartime <- function(dat, lat, lon, tz, ...,
+                      tryFormats=c("%Y-%m-%d %H:%M:%OS",
+                                   "%Y/%m/%d %H:%M:%OS",
+                                   "%Y:%m:%d %H:%M:%OS",
+                                   "%Y-%m-%d %H:%M",
+                                   "%Y/%m/%d %H:%M",
+                                   "%Y:%m:%d %H:%M",
+                                   "%Y-%m-%d",
+                                   "%Y/%m/%d",
+                                   "%Y:%m:%d")){
+  dat <- as.POSIXlt(dat, tryFormats=tryFormats, ...)
+  posdat <- list(lat,lon,tz)
+  if(!all(unlist(lapply(posdat, inherits, "numeric"))) |
      any(unlist(lapply(posdat, length)) != length(dat) &
          unlist(lapply(posdat, length)) != 1))
-    stop("lat, long and tz must all be numeric scalars or vectors the same length as dat")
+    stop("lat, lon and tz must all be numeric scalars, or vectors the same length as dat")
 
-  suntimes <- wrap(insol::daylength(lat, long, insol::JD(dat), tz)[,-3] * pi/12)
+  suntimes <- wrap(get_suntimes(dat, lat, lon, tz)[,-3] * pi/12)
   tm <- gettime(dat)
   list(input=dat, clock=tm, solar=transtime(tm, suntimes))
 }
@@ -1020,8 +1099,7 @@ solartime <- function(dat, lat, long, tz, format="%Y-%m-%d %H:%M:%S"){
 #' @examples
 #' data(BCItime)
 #' subdat <- subset(BCItime, species=="ocelot")
-#' jdate <- insol::JD(strptime(subdat$date, "%d/%m/%Y %H:%M", "UTC"))
-#' suntimes <- pi/12 * insol::daylength(9.156335, -79.847682, jdate, -5)[, -3]
+#' suntimes <- pi/12 * get_suntimes(subdat$date, 9.156335, -79.847682, -5)[, -3]
 #' rawtimes <- subdat$time*2*pi
 #' avgtimes <- transtime(rawtimes, suntimes)
 #' eqntimes <- transtime(rawtimes, suntimes, type="equinoctial")
